// VotingScreen.jsx
import { useState, useEffect, useMemo } from 'react';
import { useGame } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faVoteYea, faTrophy, faMusic, faCheck, faMedal, faCrown } from '@fortawesome/free-solid-svg-icons';
import YouTubeEmbed from './YouTubeEmbed';

function VotingScreen() {
  const { lobby, currentPlayer, submitVote, isHost } = useGame();
  const [hasVoted, setHasVoted] = useState(false);
  const [selectedSong, setSelectedSong] = useState(null);
  const [countdown, setCountdown] = useState(null);
  const [processingByeAdvance, setProcessingByeAdvance] = useState(false);
  
  // Hole aktuellen Kampf
  const battle = lobby?.currentBattle || null;

  // Berechne die Turnierphase basierend auf der Rundennummer und Gesamtliedern
  const tournamentPhase = useMemo(() => {
    if (!lobby || !battle) return '';
    
    // Hole Gesamtanzahl der Lieder im Turnier
    const totalSongs = lobby.songs?.length || 0;
    
    if (totalSongs === 0) return 'Vorrunden';
    
    // Berechne die insgesamt benötigten Runden für das Turnier
    const totalRounds = Math.ceil(Math.log2(totalSongs));
    const currentRound = battle.round + 1;
    const roundsRemaining = totalRounds - currentRound;
    
    if (roundsRemaining === 0) return 'Finale';
    if (roundsRemaining === 1) return 'Halbfinale';
    if (roundsRemaining === 2) return 'Viertelfinale';
    return 'Vorrunde';
  }, [lobby, battle]);

  // Prüfe, ob der Spieler bereits abgestimmt hat
  useEffect(() => {
    if (battle && battle.votes && currentPlayer) {
      // Prüfe, ob die ID des Spielers im Stimmen-Objekt existiert
      // Da Stimmen als Objekt, nicht als Map gesendet werden
      const votesObj = battle.votes || {};
      setHasVoted(Object.prototype.hasOwnProperty.call(votesObj, currentPlayer.id) || 
                  Object.keys(votesObj).includes(currentPlayer.id));
    } else {
      setHasVoted(false);
    }
  }, [battle, currentPlayer]);

  // Behandle Stimmenwahl
  const handleVoteSelect = (songId) => {
    if (hasVoted) return;
    
    setSelectedSong(songId);
  };

  // Sende endgültige Stimme
  const handleSubmitVote = async () => {
    if (!selectedSong || hasVoted) return;
    
    await submitVote(selectedSong);
    // setHasVoted wird jetzt durch den useEffect behandelt, der die Stimmen prüft
  };
  
  // Handle bye round advancement - für automatisches Weiterkommen
  const handleByeAdvance = async () => {
    if (processingByeAdvance || !isHost) return;
    
    setProcessingByeAdvance(true);
    try {
      // Nur der Host kann im Bye-Modus weiterschaltenNNULL
      if (battle && battle.song1 && !battle.song2 && battle.song1.id) {
        await submitVote(battle.song1.id);
      }
    } catch (error) {
      console.error("Fehler beim Fortfahren:", error);
    } finally {
      // Verzögerung, um mehrere Klicks zu verhindern
      setTimeout(() => setProcessingByeAdvance(false), 1000);
    }
  };

  // Hole YouTube-Video-IDs aus Links
  const getYouTubeId = (url) => {
    if (!url) return null;
    
    const regExp = /^.*(youtu.be\/|v\/|u\/\w\/|embed\/|watch\?v=|&v=)([^#&?]*).*/;
    const match = url.match(regExp);
    
    return (match && match[2].length === 11) ? match[2] : null;
  };

  if (!battle || !battle.song1) {
    return (
      <div className="voting-screen">
        <h2>Bereite den nächsten Kampf vor...</h2>
      </div>
    );
  }
  
  // Behandle "Freilos"-Runden, in denen ein Lied automatisch weiterkommt
  if (battle.bye === true && battle.song1 && !battle.song2) {
    const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
    
    return (
      <div className="voting-screen">
        <header className="voting-header">
          <h1>
            <FontAwesomeIcon icon={faTrophy} /> Automatisches Weiterkommen
          </h1>
          <div className="round-info">
            <span>Runde {battle.round + 1}</span>
          </div>
        </header>
        
        <div className="bye-container">
          <div className="song-card bye-winner">
            <div className="song-spotlight"></div>
            <div className="song-details">
              <h3>{battle.song1.title}</h3>
              <p>{battle.song1.artist}</p>
              <div className="auto-advance-notice">
                <p>Dieses Lied kommt automatisch in die nächste Runde!</p>
                <p className="host-info">{isHost ? 'Du kannst als Host zum nächsten Kampf weitergehen.' : 'Warte auf den Host, um fortzufahren.'}</p>
              </div>
            </div>
            
            {song1Id ? (
              <div className="video-container">
                <YouTubeEmbed videoId={song1Id} />
              </div>
            ) : (
              <div className="no-video">
                <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
                <span>Kein Video verfügbar</span>
              </div>
            )}
          </div>
        </div>
        
        <div className="voting-status">
          {isHost ? (
            <button 
              className={`btn primary pixelated full-width ${processingByeAdvance ? 'disabled' : ''}`} 
              onClick={handleByeAdvance}
              disabled={processingByeAdvance}
            >
              {processingByeAdvance ? 'Wird geladen...' : 'Mach weiter'}
              <span className="pixel-corner tl"></span>
              <span className="pixel-corner tr"></span>
              <span className="pixel-corner bl"></span>
              <span className="pixel-corner br"></span>
            </button>
          ) : (
            <p className="auto-advance-notice">Warte auf den Host, um fortzufahren...</p>
          )}
        </div>
      </div>
    );
  }

  const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
  const song2Id = getYouTubeId(battle.song2?.youtubeLink || '');

  return (
    <div className="voting-screen">
      <header className="voting-header">
        <h1>
          <FontAwesomeIcon icon={tournamentPhase === 'Finale' ? faCrown : tournamentPhase === 'Halbfinale' ? faMedal : faVoteYea} /> 
          {tournamentPhase}
        </h1>
        <div className="round-info">
          <span>Runde {battle.round + 1}</span>
          {hasVoted && <span className="voted-badge">Du hast abgestimmt!</span>}
        </div>
      </header>
      
      <div className="battle-container">
        <div 
          className={`song-card ${selectedSong === battle.song1.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song1.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song1.title}</h3>
            <p>{battle.song1.artist}</p>
          </div>
          
          {song1Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song1Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>Kein Video verfügbar</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song1Votes}</span>
              <span className="vote-text">Stimmen</span>
            </div>
          )}
          
          {selectedSong === battle.song1.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
        
        <div className="versus">
          <span className="versus-text">VS</span>
          <div className="versus-decoration"></div>
        </div>
        
        <div 
          className={`song-card ${selectedSong === battle.song2.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song2.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song2.title}</h3>
            <p>{battle.song2.artist}</p>
          </div>
          
          {song2Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song2Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>Kein Video verfügbar</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song2Votes}</span>
              <span className="vote-text">Stimmen</span>
            </div>
          )}
          
          {selectedSong === battle.song2.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
      </div>
      
      {!hasVoted && (
        <div className="voting-actions">
          <button 
            className="btn primary" 
            onClick={handleSubmitVote}
            disabled={!selectedSong}
          >
            Abstimmen
          </button>
        </div>
      )}
      
      <div className="voting-status">
        <p>{hasVoted ? 'Warte auf andere Spieler...' : 'Wähle deinen Favoriten!'}</p>
        <div className="votes-count">
          <span>{battle.voteCount || 0}</span> von <span>{lobby?.players?.filter(p => p.isConnected).length || 0}</span> Stimmen
        </div>
        
        {/* Liste der Spielerstimmen */}
        <div className="player-votes">
          <h4>Abstimmende</h4>
          <ul className="players-voted-list">
            {lobby?.players?.filter(p => p.isConnected).map(player => {
              // Prüfe, ob dieser Spieler abgestimmt hat
              const hasPlayerVoted = battle.votes && 
                Object.keys(battle.votes).includes(player.id);
              
              return (
                <li key={player.id} className={hasPlayerVoted ? 'voted' : 'not-voted'}>
                  {player.name} {player.id === currentPlayer.id && '(Du)'}
                  {hasPlayerVoted && 
                    <FontAwesomeIcon icon={faCheck} className="vote-icon" />
                  }
                </li>
              );
            })}
          </ul>
        </div>
      </div>
    </div>
  );
}

export default VotingScreen;
